@@ -18,13 +18,8 @@ License: Apache-2.0
     
     
 ToDo:
-   Add tests for dict, yaml, and toml (and figure out optional import of yaml
-    and toml dependencies)
    
    
-"""
-
-"""
 For Developers:
 
 As with all of my packages, I use Google-style docstrings and follow the Google 
@@ -50,7 +45,7 @@ any other package maintainer - I want to ensure the package is as accessible and
 useful as possible.
      
 """
-__version__ = '0.1.1'
+__version__ = '0.1.4'
 
 __package__ = 'bobbie'
 
@@ -58,3 +53,5 @@ __author__ = 'Corey Rayburn Yung'
 
 
 from .core import *
+from .extensions import *
+from .parsers import *
\ No newline at end of file
