@@ -17,7 +17,7 @@ License: Apache-2.0
     limitations under the License.
 
 Contents:
-    Settings (amos.Dictionary, amos.SourceFactory): stores configuration 
+    Settings (camina.Dictionary, ashford.SourceFactory): stores configuration 
         settings after either loading them from disk or by the passed arguments.
 
 ToDo:
@@ -32,13 +32,16 @@ import dataclasses
 import importlib
 import importlib.util
 import pathlib
-from typing import Any, ClassVar, Optional, Type, Union
+from typing import Any, Optional, TYPE_CHECKING
 
-import amos
+import camina
 
+if TYPE_CHECKING:
+     from . import extensions
+     
 
 @dataclasses.dataclass
-class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
+class Settings(camina.Dictionary): 
     """Loads and stores configuration settings.
 
     To create settings instance, a user can pass as the 'contents' parameter a:
@@ -69,25 +72,28 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
         contents (MutableMapping[Hashable, Any]): a dict for storing 
             configuration options. Defaults to en empty dict.
         default_factory (Optional[Any]): default value to return when the 'get' 
-            method is used. Defaults to an empty dict.
-        default (Mapping[str, Mapping[str]]): any default options that should
-            be used when a user does not provide the corresponding options in 
-            their configuration settings. Defaults to an empty dict.
-        infer_types (bool): whether values in 'contents' are converted to other 
-            datatypes (True) or left alone (False). If 'contents' was imported 
-            from an .ini file, all values will be strings. Defaults to True.
+            method is used. Defaults to an empty camina.Dictionary.
+        defaults (Optional[Mapping[str, Mapping[str]]]): any default options 
+            that should be used when a user does not provide the corresponding 
+            options in their configuration settings. Defaults to an empty dict.
+        infer_types (Optional[bool]): whether values in 'contents' are converted 
+            to other datatypes (True) or left alone (False). If 'contents' was 
+            imported from an .ini file, all values will be strings. Defaults to 
+            True.
+        parsers (Optional[MutableMapping[Hashable, extensions.Parser]]): keys 
+            are str names of Parser instances and the values are Parser 
+            instances. The keys will be used as attribute names when the 'parse'
+            method is automatically called if 'parsers' is not None. Defaults to 
+            None.
 
     """
     contents: MutableMapping[Hashable, Any] = dataclasses.field(
         default_factory = dict)
-    default_factory: Optional[Any] = dict
-    default: Mapping[Hashable, Any] = dataclasses.field(
+    default_factory: Optional[Any] = camina.Dictionary
+    defaults: Optional[Mapping[Hashable, Any]] = dataclasses.field(
         default_factory = dict)
-    infer_types: bool = True
-    sources: ClassVar[Mapping[Type, str]] = {
-        MutableMapping: 'dictionary', 
-        pathlib.Path: 'path',  
-        str: 'path'}
+    infer_types: Optional[bool] = True
+    parsers: Optional[MutableMapping[Hashable, extensions.Parser]] = None
 
     """ Initialization Methods """
 
@@ -95,23 +101,46 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
         """Initializes class instance attributes."""
         # Calls parent and/or mixin initialization method(s).
         with contextlib.suppress(AttributeError):
-            super().__post_init__(*args, **kwargs) # type: ignore
+            super().__post_init__() 
         # Converts 'contents' if it is not a dict.
-        if not (self.contents, Mapping):
+        if not (self.contents, MutableMapping):
             self = self.create(
                 item = self.contents,
                 default_factory = self.default_factory,
-                default = self.default,
-                infer_types = self.infer_types)
+                defaults = self.defaults,
+                infer_types = self.infer_types,
+                parsers = self.parsers)
         # Infers types for values in 'contents', if the 'infer_types' option is 
         # selected.
         if self.infer_types:
             self.contents = self._infer_types(contents = self.contents)
         # Adds default settings as backup settings to 'contents'.
-        self.contents = self._add_default(contents = self.contents)
+        self.contents = self._add_defaults(contents = self.contents)
+        # Adds descriptora from 'parsers',
+        self.parse()
 
     """ Class Methods """
 
+    @classmethod
+    def create(cls, source: Any, **kwargs: Any) -> Settings:
+        """Calls corresponding creation class method to instance a class.
+
+        Raises:
+            TypeError: if 'source' is not a str, pathlib.Path, or dict-like 
+                object.
+
+        Returns:
+            Settings: instance of a Settings.
+            
+        """
+        if isinstance(source, (str, pathlib.Path)):
+            return cls.from_path(source = source, **kwargs)
+        elif isinstance(source, MutableMapping):
+            return cls.from_dictionary(source = source, **kwargs)
+        else:
+            raise TypeError(
+                f'source must be a str, Path, or dict-like object')
+
     @classmethod
     def from_dictionary(
         cls, 
@@ -132,19 +161,19 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
     @classmethod
     def from_path(
         cls, 
-        source: Union[str, pathlib.Path], 
+        source: str | pathlib.Path, 
         **kwargs: Any) -> Settings:
         """[summary]
 
         Args:
-            source (Union[str, pathlib.Path]): path to file with settings to 
-                store in a Settings instance.
+            source (str | pathlib.Path): path to file with settings to store in 
+                a Settings instance.
                 
         Returns:
             Settings: an instance derived from 'source'.
             
         """
-        path = amos.pathlibify(item = source)   
+        path = camina.pathlibify(item = source)   
         extension = path.suffix[1:]
         load_method = getattr(cls, f'from_{extension}')
         return load_method(source = path, **kwargs)
@@ -152,13 +181,13 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
     @classmethod
     def from_ini(
         cls, 
-        source: Union[str, pathlib.Path], 
+        source: str | pathlib.Path, 
         **kwargs: Any) -> Settings:
         """Returns settings from an .ini file.
 
         Args:
-            source (Union[str, pathlib.Path]): path to file with settings to 
-                store in a Settings instance.
+            source (str | pathlib.Path): path to file with settings to store in 
+                a Settings instance.
 
         Returns:
             Settings: an instance derived from 'source'.
@@ -167,7 +196,7 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
             FileNotFoundError: if the path does not correspond to a file.
 
         """
-        path = amos.pathlibify(item = source) 
+        path = camina.pathlibify(item = source) 
         if 'infer_types' not in kwargs:
             kwargs['infer_types'] = True
         try:
@@ -181,13 +210,13 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
     @classmethod
     def from_json(
         cls,
-        source: Union[str, pathlib.Path], 
+        source: str | pathlib.Path, 
         **kwargs: Any) -> Settings:
         """Returns settings from an .json file.
 
         Args:
-            source (Union[str, pathlib.Path]): path to file with settings to 
-                store in a Settings instance.
+            source (str | pathlib.Path): path to file with settings to store in 
+                a Settings instance.
 
         Returns:
             Settings: an instance derived from 'source'.
@@ -197,7 +226,7 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
 
         """
         import json
-        path = amos.pathlibify(item = source) 
+        path = camina.pathlibify(item = source) 
         if 'infer_types' not in kwargs:
             kwargs['infer_types'] = True
         try:
@@ -210,14 +239,14 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
     @classmethod
     def from_py(
         cls,
-        source: Union[str, pathlib.Path], 
+        source: str | pathlib.Path, 
         **kwargs: Any) -> Settings:
         """Returns a settings dictionary from a .py file.
 
         Args:
-            source (Union[str, pathlib.Path]): path to file with settings to 
-                store in a Settings instance. The path to a python module must
-                have a '__dict__' defined and an attribute named 'settings' that 
+            source (str | pathlib.Path): path to file with settings to store in 
+                a Settings instance. The path to a python module must have a 
+                '__dict__' defined and an attribute named 'settings' that 
                 contains the settings to use for creating an instance.
 
         Returns:
@@ -228,9 +257,8 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
                 file.
 
         """
-        path = amos.pathlibify(item = source) 
-        if 'infer_types' not in kwargs:
-            kwargs['infer_types'] = False
+        path = camina.pathlibify(item = source) 
+        kwargs['infer_types'] = False
         try:
             path = pathlib.Path(path)
             import_path = importlib.util.spec_from_file_location(
@@ -245,13 +273,13 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
     @classmethod
     def from_toml(
         cls,
-        source: Union[str, pathlib.Path], 
+        source: str | pathlib.Path, 
         **kwargs: Any) -> Settings:
         """Returns settings from a .toml file.
 
         Args:
-            source (Union[str, pathlib.Path]): path to file with settings to 
-                store in a Settings instance.
+            source (str | pathlib.Path): path to file with settings to store in 
+                a Settings instance.
 
         Returns:
             Settings: an instance derived from 'source'.
@@ -261,7 +289,7 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
 
         """
         import toml
-        path = amos.pathlibify(item = source) 
+        path = camina.pathlibify(item = source) 
         if 'infer_types' not in kwargs:
             kwargs['infer_types'] = True
         try:
@@ -272,13 +300,13 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
     @classmethod
     def from_yaml(
         cls, 
-        source: Union[str, pathlib.Path], 
+        source: str | pathlib.Path, 
         **kwargs: Any) -> Settings:
         """Returns settings from a .yaml file.
 
         Args:
-            source (Union[str, pathlib.Path]): path to file with settings to 
-                store in a Settings instance.
+            source (str | pathlib.Path): path to file with settings to store in 
+                a Settings instance.
 
         Returns:
             Settings: an instance derived from 'source'.
@@ -288,23 +316,30 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
 
         """
         import yaml
-        path = amos.pathlibify(item = source) 
-        if 'infer_types' not in kwargs:
-            kwargs['infer_types'] = False
+        path = camina.pathlibify(item = source) 
+        kwargs['infer_types'] = False
         try:
             with open(path, 'r') as config:
                 return cls(contents = yaml.safe_load(config, **kwargs))
         except FileNotFoundError:
             raise FileNotFoundError(f'settings file {path} not found')
         
-    """ Public Methods """
+    """ Instance Methods """
 
-    def add(self, section: Hashable, contents: Mapping[Hashable, Any]) -> None:
-        """Adds 'settings' to 'contents'.
+    def add(
+        self, 
+        section: Hashable, 
+        contents: MutableMapping[Hashable, Any]) -> None:
+        """Adds 'section' to 'contents'.
+        
+        If 'section' is already a key in 'contents', the contents associated
+        with that key are updated. If 'section' doesn't exist, a new key/value
+        pair is added to 'contents'.
 
         Args:
             section (Hashable): name of section to add 'contents' to.
-            contents (Mapping[Hashable, Any]): a dict to store in 'section'.
+            contents (MutableMapping[Hashable, Any]): a dict to store in 
+            'section'.
 
         """
         try:
@@ -316,7 +351,7 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
     def inject(
         self, 
         instance: object,
-        additional: Optional[Union[Sequence[str], str]] = None,
+        additional: Optional[Sequence[str] | str] = None,
         overwrite: bool = False) -> object:
         """Injects appropriate items into 'instance' from 'contents'.
         
@@ -325,8 +360,8 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
         attributes.
 
         Args:
-            instance (object): amos class instance to be modified.
-            additional (Union[Sequence[str], str]]): other section(s) in 
+            instance (object): class instance to be modified.
+            additional (Optional[Sequence[str] | str]): other section(s) in 
                 'contents' to inject into 'instance'. Defaults to None.
             overwrite (bool]): whether to overwrite a local attribute in 
                 'instance' if there are existing values stored in that 
@@ -342,7 +377,7 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
         except AttributeError:
             pass
         if additional:
-            sections.extend(amos.iterify(additional))
+            sections.extend(camina.iterify(additional))
         for section in sections:
             try:
                 for key, value in self.contents[section].items():
@@ -353,9 +388,35 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
             except KeyError:
                 pass
         return instance
+
+    def parse(self) -> None:
+        """Adds key/value pairs in 'parsers' as class attributes."""
+        if self.parsers:
+            for key, parser in self.parsers.items():
+                setattr(self.__class__, key, parser)
+        return self
        
     """ Private Methods """
 
+    def _add_defaults(
+        self, 
+        contents: MutableMapping[Hashable, Any]) -> (
+            MutableMapping[Hashable, Any]):
+        """Creates a backup set of mappings for bobbie settings lookup.
+
+
+        Args:
+            contents (MutableMapping[Hashable, Any]): a nested contents dict to 
+                add default to.
+
+        Returns:
+            MutableMapping[Hashable, Any]: with stored default added.
+
+        """
+        new_contents = self.defaults
+        new_contents.update(contents)
+        return new_contents
+
     def _infer_types(
         self, 
         contents: MutableMapping[Hashable, Any]) -> (
@@ -375,30 +436,11 @@ class Settings(amos.Dictionary, amos.SourceFactory): # type: ignore
         for key, value in contents.items():
             if isinstance(value, dict):
                 inner_bundle = {
-                    inner_key: amos.typify(inner_value)
+                    inner_key: camina.typify(inner_value)
                     for inner_key, inner_value in value.items()}
                 new_contents[key] = inner_bundle
             else:
-                new_contents[key] = amos.typify(value)
-        return new_contents
-
-    def _add_default(
-        self, 
-        contents: MutableMapping[Hashable, Any]) -> (
-            MutableMapping[Hashable, Any]):
-        """Creates a backup set of mappings for bobbie settings lookup.
-
-
-        Args:
-            contents (MutableMapping[Hashable, Any]): a nested contents dict to 
-                add default to.
-
-        Returns:
-            MutableMapping[Hashable, Any]: with stored default added.
-
-        """
-        new_contents = self.default
-        new_contents.update(contents)
+                new_contents[key] = camina.typify(value)
         return new_contents
 
     """ Dunder Methods """
